// $Id$

//=============================================================================
/**
 *  @file  filebase.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_FILEBASE_H
#define JYUGEM_BEE_FILEBASE_H

#include "jyugem/bee.h"

namespace jyugem { namespace bee
{

//
// File I/O base class
//
class BEE_Export filebase
{
public:
  filebase(const std::string& dir, const std::string& fname,
       const std::string& ext) : m_dir(dir), m_fname(fname), m_ext(ext)
  {
    filebase::make_pathname(dir, fname, ext, m_path);
  }
  virtual ~filebase() {}

  filebase& operator=(const filebase& rhs)
  {
    if (this == &rhs)
      return *this;
    this->copy(rhs);
    return *this;
  }

  const std::string& path() const { return m_path; }

  virtual int  open(void * = null) { return 0; }
  virtual int  close()       { return 0; }
  virtual int  read(BYTE * addr, BYTE * buf, size_t words) = 0;
  virtual int  write(BYTE * buf, size_t words) = 0;
  virtual size_t size() const = 0;

  static int make_pathname(const std::string& dir, const std::string& fname,
               const std::string& ext, std::string& pathname);
  void dump() const;

//
protected:
  filebase() : m_dir(_TX("")), m_fname(_TX("")), m_ext(""), m_path(_TX("")) {}
  void copy(const filebase& rhs)
  {
    m_dir = rhs.m_dir;
    m_fname = rhs.m_fname;
    m_ext = rhs.m_ext;
    m_path = rhs.m_path;
  }

// member variables
protected:
  std::string  m_dir;    // directory that make mmap file.
  std::string  m_fname;    // filename that make mmap file.
  std::string  m_ext;    // mmap file's extention
  std::string  m_path;     // mmap file's path
};


} } // *namespace* jyugem::bee

#endif  // JYUGEM_BEE_FILEBASE_H
