// $Id$

//=============================================================================
/**
 *  @file  memory_map.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *  Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_MEMORY_MAP_H
#define JYUGEM_BEE_MEMORY_MAP_H

#include "jyugem/bee/filebase.h"
#include "ace/Mem_Map.h"


namespace jyugem { namespace bee
{

//
// Management of memory mapped file
//
class BEE_Export memory_map : public filebase
{
public:
  memory_map(const std::string& dir, const std::string& fname,
         const std::string& ext, bool clear = false, int maxq = 0)
    : filebase(dir, fname, ext), m_max(maxq) { this->map(clear); }
  virtual ~memory_map() { m_mmap.sync(); m_mmap.close(); }

  memory_map& operator=(const memory_map& rhs)
  {
    if (this == &rhs)
      return *this;
    this->copy(rhs);
    return *this;
  }

  virtual int  open(void * = null) { return 0; }
  virtual int  close()       { return 0; }
  virtual int  read(BYTE * addr, BYTE * buf, size_t words);
  virtual int  write(BYTE * buf, size_t words);
  virtual size_t size() const { return m_mmap.size(); }

  int  map(bool clear = false);
  BYTE * top()  { return (BYTE *)m_mmap.addr(); }
  void flush()  { m_mmap.sync(); }

  void dump() const;

//
protected:
  memory_map() : filebase(_TX(""), _TX(""), _TX(".map")), m_max(0) {}
  void copy(const memory_map& rhs);

// member variables
protected:
  size_t  m_max;       // max size
  BYTE *  m_current;
  ACE_Mem_Map m_mmap;

};


} } // *namespace* jyugem::bee

#endif  // JYUGEM_BEE_MEMORY_MAP_H
