// $Id$

//=============================================================================
/**
 *  @file    job_manager.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================


#ifdef _MSC_VER
#pragma warning( disable: 4267 )
#endif

#include <boost/mpl/list.hpp>
#include <boost/format.hpp>


#include <ctime>
#include <iostream>
#include <string>

#include "jyugem/bee/task.h"

namespace bee = jyugem::bee;


//
namespace jyugem { namespace gem
{

//
// Job manager class
//
template<class T>
class BEE_Export job_manager : bee::task
{
    friend class T;
    typedef ACE_Mutex_Lock             Mutex;
    typedef ACE_Guard<ACE_Mutex_Lock>  Guard;
public:
    //typedef std::map<std::string, T *, std::less<std::string> >
    //    Jobs;

    job_manager() : bee::task() {}
    virtual ~job_manager() {}

    virtual int notify(bee::trigger * trig, bee::task * tsk) = 0;

protected:
    T * find(const std::string& jobid)
    {
        Guard lock(m_lock);
        while (lock.locked() == 0)
            ;   // loop of guard

        std::list<T *>iterator iter = m_queue.begin();
        for ( ; iter != m_queue.end(); ++iter)
        {
            if (jobid == *iter->name())
            {
                return *iter;
            }
        }
        return null;
    }

    int append(T * job)
    {
        Guard lock(m_lock);
        while (lock.locked() == 0)
            ;   // loop guard
        m_queue.push_back(job);
        return 0;
    }

    int hoq(T * job)
    {
        Guard lock(m_lock);
        while (lock.locked() == 0)
            ;   // loop guard
        m_queue.push_front(job);
        return 0;
    }

    T * remove(const std::string& jobid)
    {
        Guard lock(m_lock);
        while (lock.locked() == 0)
            ;   // loop of guard

        std::list<T *>iterator iter = m_queue.begin();
        for ( ; iter != m_queue.end(); ++iter)
        {
            if (jobid == *iter->name())
            {
                T * job = *iter;
                m_queue.erase(iter);
                return job;
            }
        }
        return null;
    }

//
protected:
    Mutex          m_lock;
    std::list<T *> m_queue;   // Job-T queue
};


} }  // *namespace* jyugem::gem

