// $Id$

//=============================================================================
/**
 *  @file    log_manager.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "jyugem/bee/log_manager.h"
using namespace jyugem::bee;

#include "ace/Log_Msg.h"
#include "ace/Log_Record.h"
#include "ace/Log_Msg_Callback.h"
#include "ace/Singleton.h"
#include "ace/Trace.h"

namespace jyugem { namespace bee
{

//
// Timestamp that is Callback functor class for Logging
//
//  Calling:
//
//    const ACE_TCHAR * program = ACE_TEXT("Program Name");
//    ACE_LOG_MSG->open(program);
//    cb_timestamp * callback = new cb_timestamp();
//    ACE_LOG_MSG->set_flags(ACE_Log_Msg::MSG_CALLBACK);
//    ACE_LOG_MSG->msg_callback(callback);
//
//    * Must set callback for each threads
//
class cb_timestamp : public ACE_Log_Msg_Callback
{
public:
  void log(ACE_Log_Record& log_record)
  {
    u_long verbose_flag = log_record.type();
    if (! (ACE_BIT_ENABLED(verbose_flag, ACE_Log_Msg::VERBOSE) ||
           ACE_BIT_ENABLED(verbose_flag, ACE_Log_Msg::VERBOSE_LITE)))
    {
      ACE_TCHAR timestamp[26];
      ACE_TCHAR ctp[26]; // 26 is a magic number...

      time_t now = log_record.time_stamp().sec();
      if (ACE_OS::ctime_r(&now, ctp, sizeof(ctp)) == 0)
      {
        return ;
      }

      /* 01234567890123456789012345 */
      /* Wed Oct 18 14:25:36 1989n0 */

      ctp[19] = '\0'; // NUL-terminate after the time.
      ACE_OS::sprintf(timestamp, ACE_LIB_TEXT("%s.%03ld "),
                      ctp + 11,
                      ((long)log_record.time_stamp().usec()) / 1000);

      /*  01234567890123 */
      /* "14:25:36.543 " */
      std::string buf(timestamp);
      buf += log_record.msg_data();
      log_record.msg_data(buf.c_str());

      //// Can't call round_up()
      // ACE_TCHAR * top = const_cast<ACE_TCHAR *>(log_record.msg_data());
      // size_t q = ACE_OS::strlen(top);
      // if (q > ACE_Log_Record::MAXLOGMSGLEN)
      //   q = ACE_Log_Record::MAXLOGMSGLEN;
      // ACE_OS::memmove(top + 13, top, q * sizeof(ACE_TCHAR));
      // ACE_OS::memmove(top, timestamp, 13 * sizeof(ACE_TCHAR));
      // *(top + q + 13) = '\0';
      // log_record.length(log_record.msg_data_len());
    }
  }
};

typedef ACE_Singleton<cb_timestamp, ACE_Null_Mutex> TimestampCallback;
//
// Calling:
//   cb_timestamp * callback = TimestampCallback::instance();
//

//-----------------------------------------------------------------------------
// Log Manager
//

// Open logger (static)
void log_manager::start(const BCHAR * progname)
{
  BCHAR * logname = (progname == null) ? _TX("JYUGEM") : progname;

  if (ACE_LOG_MSG->open(logname, ACE_Log_Msg::LOGGER, ACE_DEFAULT_LOGGER_KEY)
      != -1)
  {
    std::cout << "Logging open error\n";

    ACE_Trace::stop_tracing();   // disable trace

    // ACE_LOG_MSG->set_flags(ACE_Log_Msg::MSG_CALLBACK);
    // ACE_LOG_MSG->msg_callback(TimestampCallback::instance());
  }
  else
  {
    std::cout << "Logging start\n";
  }
}

// Start logger in thread (static)
void log_manager::start_thread()
{
  // ACE_LOG_MSG->set_flags(ACE_Log_Msg::MSG_CALLBACK);
  // ACE_LOG_MSG->msg_callback(TimestampCallback::instance());
}

// Enable the LM_DEBUG messages at the process level. (static)
void log_manager::enable_debug()
{
  u_long priority = ACE_LOG_MSG->priority_mask(ACE_Log_Msg::PROCESS);
  ACE_SET_BITS(priority, LM_DEBUG);
  ACE_LOG_MSG->priority_mask(priority, ACE_Log_Msg::PROCESS);
}

// Disable the LM_DEBUG messages at the process level. (static)
void log_manager::disable_debug()
{
  u_long priority = ACE_LOG_MSG->priority_mask(ACE_Log_Msg::PROCESS);
  ACE_CLR_BITS(priority, LM_DEBUG);
  ACE_LOG_MSG->priority_mask(priority, ACE_Log_Msg::PROCESS);
}

// Enable the LM_INFO messages at the process level. (static)
void log_manager::enable_info()
{
  u_long priority = ACE_LOG_MSG->priority_mask(ACE_Log_Msg::PROCESS);
  ACE_SET_BITS(priority, LM_INFO);
  ACE_LOG_MSG->priority_mask(priority, ACE_Log_Msg::PROCESS);
}

// Disable the LM_INFO messages at the process level. (static)
void log_manager::disable_info()
{
  u_long priority = ACE_LOG_MSG->priority_mask(ACE_Log_Msg::PROCESS);
  ACE_CLR_BITS(priority, LM_INFO);
  ACE_LOG_MSG->priority_mask(priority, ACE_Log_Msg::PROCESS);
}

} }  // *namespace* - jyugem::bee
