// $Id$

//=============================================================================
/**
 *  @file  meta_template.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_META_TEMPLATE_H
#define JYUGEM_BEE_META_TEMPLATE_H

#include "jyugem/bee/metadata.h"

namespace jyugem { namespace bee
{

//
// Meta Data class
//
template<typename T>
class BEE_Export metadata_t : public metadata
{
public:
  // Constructor
  metadata_t(int type, size_t bytes, const std::string& name)
    : metadata(type, bytes, name) {}

  virtual ~metadata_t() {}

  virtual bool equal(const value& lhs, const value& rhs);
  virtual bool less(const value& lhs, const value& rhs);

  virtual size_t length(const value& v) const;
  virtual bool getBool(const value& v) const;
  virtual int  getInt(const value& v) const;
  virtual UINT getUInt(const value& v) const;
  virtual int  setInt(value& lv, int v);

  virtual int  set(value& lhs, const value& rhs);
  virtual int  set(value& lhs, const std::string& vs);
  virtual int  set(value& lhs, const BYTE * buf, size_t q,
           int order = ENDIAN);
  virtual int  get(const value& v, std::string& retstr);
  virtual int  get(const value& v, BYTE * buf, size_t q,
           int order = ENDIAN);

  virtual int  make(value& outv);
//
protected:
  virtual void copy(value& lhs, const value& rhs);
};


template<class T>
class BEE_Export metaplural_t : public metadata
{
public:
  // Constructor
  metaplural_t(int type, size_t bytes, const std::string& name)
    : metadata(type, bytes, name) {}

  virtual ~metaplural_t() {}

  virtual bool equal(const value& lhs, const value& rhs);
  virtual bool less(const value& lhs, const value& rhs);

  virtual size_t length(const value& v) const;
  virtual bool getBool(const value& v) const;
  virtual int  getInt(const value& v) const;
  virtual UINT getUInt(const value& v) const;
  virtual int  setInt(value& lv, int v);

  virtual int  set(value& lhs, const value& rhs);
  virtual int  set(value& lhs, const std::string& vs);
  virtual int  set(value& lhs, const BYTE * buf, size_t q,
           int order = ENDIAN);
  virtual int  get(const value& v, std::string& retstr);
  virtual int  get(const value& v, BYTE * buf, size_t q,
           int order = ENDIAN);

  virtual int  make(value& outv);
//
protected:
  virtual void copy(value& lhs, const value& rhs);
};

template<class T>
class BEE_Export metavalue_t : public metadata
{
public:
  // Constructor
  metavalue_t(int type, size_t bytes, const std::string& name)
    : metadata(type, bytes, name) {}

  virtual ~metavalue_t() {}

  virtual bool equal(const value& lhs, const value& rhs);
  virtual bool less(const value& lhs, const value& rhs);

  virtual size_t length(const value& v) const;
  virtual bool getBool(const value& v) const;
  virtual int  getInt(const value& v) const;
  virtual UINT getUInt(const value& v) const;
  virtual int  setInt(value& lv, int v);

  virtual int  set(value& lhs, const value& rhs);
  virtual int  set(value& lhs, const std::string& vs);
  virtual int  set(value& lhs, const BYTE * buf, size_t q,
           int order = ENDIAN);
  virtual int  get(const value& v, std::string& retstr);
  virtual int  get(const value& v, BYTE * buf, size_t q,
           int order = ENDIAN);

  virtual int  make(value& outv);
//
protected:
  virtual void copy(value& lhs, const value& rhs);
};

//
// Definition of meta bool data
//
class meta_bool : public metadata_t<bool>
{
public:
  meta_bool() : metadata_t<bool>(BEE_BOOLEAN, sizeof(bool), _TX("bool")) {}
};

//
// Definition of meta character data
//
class meta_char : public metadata_t<char>
{
public:
  meta_char() : metadata_t<char>(BEE_CHAR, sizeof(char), _TX("char")) {}
};

//
// Definition of meta byte data
//
class meta_byte : public metadata_t<bee::byte>
{
public:
  meta_byte()
    : metadata_t<bee::byte>(BEE_BYTE, sizeof(bee::byte), _TX("byte")) {}
};

//
// Definition of meta short integer data
//
class meta_short : public metadata_t<short>
{
public:
  meta_short()
    : metadata_t<short>(BEE_SHORT, sizeof(short), _TX("short")) {}
};

//
// Definition of meta unsigned short integer data
//
class meta_ushort : public metadata_t<bee::ushort>
{
public:
  meta_ushort()
    : metadata_t<bee::ushort>(BEE_USHORT, sizeof(bee::ushort), _TX("ushort")) {}
  };

//
// Definition of meta integer data
//
class meta_int : public metadata_t<int>
{
public:
  meta_int() : metadata_t<int>(BEE_INT, sizeof(int), _TX("int")) {}
};

//
// Definition of meta unsigne integer data
//
class meta_uint : public metadata_t<UINT>
{
public:
  meta_uint()
    : metadata_t<uint>(BEE_UINT, sizeof(UINT), _TX("uint")) {}
};

//
// Definition of meta long integer data
//
class meta_longlong : public metadata_t<bee::LONGLONG>
{
public:
  meta_longlong()
    : metadata_t<bee::LONGLONG>(BEE_LONGLONG, sizeof(bee::LONGLONG),
      _TX("longlong")) {}
};

//
// Definition of meta unsignef long integer data
//
class meta_ulonglong : public metadata_t<bee::ULONGLONG>
{
public:
  meta_ulonglong()
    : metadata_t<bee::ULONGLONG>(BEE_ULONGLONG, sizeof(bee::ULONGLONG),
      _TX("ulonglong")) {}
};

//
// Definition of meta float data
//
class meta_float : public metadata_t<float>
{
public:
  meta_float() : metadata_t<float>(BEE_FLOAT, sizeof(float), _TX("float")) {}
};

//
// Definition of meta double data
//
class meta_double : public metadata_t<double>
{
public:
  meta_double()
    : metadata_t<double>(BEE_DOUBLE, sizeof(double), _TX("double")) {}
};

//
// Definition of meta vector<byte> data
//
class meta_binary : public metaplural_t<binary>
{
public:
  meta_binary()
    : metaplural_t<binary>(BEE_BINARY, sizeof(bee::byte), _TX("binary")) {}
};

//
// Definition of meta string data
//
class meta_string : public metadata_t<std::string>
{
public:
  meta_string()
    : metadata_t<std::string>(BEE_STRING, sizeof(BCHAR), _TX("string")) {}
  };

//
// Definition of meta value array data
//
class meta_array : public metavalue_t<array_val>
{
public:
  meta_array()
    : metavalue_t<array_val>(BEE_ARRAY, 0, _TX("array")) {}
};

//
// Definition of meta value struct data
//
class meta_struct : public metavalue_t<struct_val>
{
public:
  meta_struct()
    : metavalue_t<struct_val>(BEE_STRUCT, 0, _TX("struct")) {}
};

//
// Definition of meta vector<char> data
//
class plural_char : public metaplural_t<array_char>
{
public:
  plural_char()
    : metaplural_t<array_char>(BEE_PLURAL_CHAR,
                   sizeof(char), _TX("char[%d]")) {}
};

//
// Definition of meta vector<short> data
//
class plural_short : public metaplural_t<array_short>
{
public:
  plural_short()
    : metaplural_t<array_short>(BEE_PLURAL_SHORT,
                  sizeof(short), _TX("short[%d]")) {}
};

//
// Definition of meta vector<ushort> data
//
class plural_ushort : public metaplural_t<array_ushort>
{
public:
  plural_ushort()
    : metaplural_t<array_ushort>(BEE_PLURAL_USHORT,
                   sizeof(ushort), _TX("ushort[%d]")) {}
};

//
// Definition of meta vector<int> data
//
class plural_int : public metaplural_t<array_int>
{
public:
  plural_int() : metaplural_t<array_int>(BEE_PLURAL_INT,
                      sizeof(int), _TX("int[%d]")) {}
};

//
// Definition of meta vector<uint> data
//
class plural_uint : public metaplural_t<array_uint>
{
public:
  plural_uint() : metaplural_t<array_uint>(BEE_PLURAL_UINT,
                       sizeof(uint), _TX("uint[%d]")) {}
};

//
// Definition of meta vector<longlong> data
//
class plural_longlong : public metaplural_t<array_longlong>
{
public:
  plural_longlong()
    : metaplural_t<array_longlong>(BEE_PLURAL_LONGLONG,
                   sizeof(bee::LONGLONG),
                   _TX("longlong[%d]")) {}
};

//
// Definition of meta vector<ulonglong> data
//
class plural_ulonglong : public metaplural_t<array_ulonglong>
{
public:
  plural_ulonglong()
    : metaplural_t<array_ulonglong>(BEE_PLURAL_ULONGLONG,
                    sizeof(bee::ULONGLONG),
                    _TX("ulonglong[%d]")) {}
};

//
// Definition of meta vector<float> data
//
class plural_float : public metaplural_t<array_float>
{
public:
  plural_float()
    : metaplural_t<array_float>(BEE_PLURAL_FLOAT,
                  sizeof(float), _TX("float[%d]")) {}
};

//
// Definition of meta vector<double> data
//
class plural_double : public metaplural_t<array_double>
{
public:
  plural_double()
    : metaplural_t<array_double>(BEE_PLURAL_DOUBLE,
                   sizeof(double), _TX("double[%d]")) {}
};


} } // *namespace*

#endif  /* JYUGEM_BEE_META_TEMPLATE_H */
