// $Id$

//=============================================================================
/**
 *  @file  task.cpp
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "jyugem/bee/task.h"
using namespace jyugem::bee;

namespace jyugem { namespace bee
{

int task::add_timer(const ACE_Time_Value& interval, timer_handler * tmh)
{
  TRACE_FUNCTION();
  int tmid = ACE_Reactor::instance()->schedule_timer(tmh, null, interval);
  if (tmid == -1)
  {
    TRACE_ERROR((_TX("[%s] Timer schedule failed"), this->name_p()));
    return -1;
  }
  // We store the id into the handler, this is only used to produce
  // nicer messages.
  tmh->set(this, tmid);

  // TRACE_DEBUG((_TX("[%s] Start timer(%d) : scheduling timer id = %d\n"),
  //       this->name_p(), term.sec(), tmid));

  return tmid;
}

int task::cancel_timer(int timer_id)   // Cancel timer <id>.
{
  TRACE_FUNCTION();
  if (timer_id < 0)
    return -1;

  ACE_Reactor::instance()->cancel_timer(timer_id);

  // TRACE_DEBUG((_TX("[%s] scheduled timer id = %d\n"), this->name_p(),
  //       timer_id));

  return 0;
}

//
// Service thread.
//
int task::svc(void)
{
  TRACE_FUNCTION();
  // TRACE_DEBUG((_TX("%s start.\n"), this->name_p()));

  int  result = 0;

  // Receive trigger request.
  for (;;)
  {
    auto_ptr<ACE_Method_Request> mo(this->m_activation_queue.dequeue());
    // Call it.
    if ((result = mo->call()) == -1)
    {
      break;
    }
    // Destructor automatically deletes it.
  }

  if (result == -1 && errno == EWOULDBLOCK)
  {
    TRACE_ERROR((_TX("(%s) %p\n"), this->name_p(),
           _TX("timed out waiting for message")));
  }

  TRACE_DEBUG((_TX("%s end.\n"), this->name_p()));
  return 0;
}


} }  // *namespace* - jyugem::bee